@@ -3,7 +3,6 @@
     utils::packageDescription("rioja", fields="Version"), appendLF = TRUE)
 }
 
-.onUnload <- function(libpath)
+.onUnload <- function(libpath) {
     library.dynam.unload("rioja", libpath)
-
-
+}
